(* fichier listp.ml : calcul du n-me entier de Hamming par liste paresseuse *)

open Lazy
open Num

(* liste paresseuse d'entiers de Hamming *)
type listp = {hd:num; tl:listp Lazy.t}

(* n-me lment d'une liste paresseuse *)
let rec nme n liste = if n = 0 then liste.hd else nme (n-1) (force liste.tl)

(* multiplie une liste par p *)
let rec mult p liste =
  {hd = liste.hd */ p;
   tl = lazy(mult p (force liste.tl))}

(* fusion sans rptition de listes paresseuses *)
let rec (@) a b = let c = compare_num a.hd b.hd in
       if c < 0 then {hd=a.hd; tl=lazy((force a.tl) @     b       )}
  else if c > 0 then {hd=b.hd; tl=lazy(    a        @ (force b.tl))}
  else               {hd=a.hd; tl=lazy((force a.tl) @ (force b.tl))}

(* calcul *)
let _ = 
  let rec h =
    {hd = (Int 1);
     tl = lazy((mult (Int 2) h) @ (mult (Int 3) h) @ (mult (Int 5) h))} in
  let n = int_of_string(Sys.argv.(1)) in
  print_string(string_of_num(nme n h));
  print_newline()
