(* fichier nocomp.ml : classement sans comparaison *)

(* Suite de Hamming : a,b,c sont les exposants de 2,3,5 *)
type listm = {a:int; b:int; c:int; mutable tl:listm}
let (=.) x y = x.a = y.a & x.b = y.b & x.c = y.c

(* multiplication et concatnation en tte *)
let double x    = {a=x.a+1; b=x.b;   c=x.c;   tl=x}
and triple x    = {a=x.a;   b=x.b+1; c=x.c;   tl=x}
and quintuple x = {a=x.a;   b=x.b;   c=x.c+1; tl=x}

(* calcul de minimum (mthode de D. Doligez *)
open Num

let valeur(x) = (Int 2)**/(Int x.a) */ (Int 3)**/(Int x.b) */ (Int 5)**/(Int x.c)
let min x y = if valeur(x) <=/ valeur(y) then x else y

let dmin x y z =
  match (y.a>0,z.b>0,x.c>0,y.c>0,x.b>0,z.a>0    ) with
      (* x<=y  y<=z  z<=x  z<=y  y<=x  x<=z  *)
        |true, _,    _,    _,    _,    true  -> x
        |true, _,    true, _,    _,    _     -> z
        |true, true, _,    _,    _,    _     -> x
        |_,    true, _,    _,    true, _     -> y
        |_,    true, true, _,    _,    _     -> y
        |_,    _,    true, true, _,    _     -> z
        |_,    _,    _,    true, true, _     -> z
        |_,    _,    _,    true, _,    true  -> x
        |_,    _,    _,    _,    true, true  -> y
                                                          
        |true, _,    _,    _,    _,    _     -> min x z
        |_,    true, _,    _,    _,    _     -> min x y
        |_,    _,    true, _,    _,    _     -> min y z
        |_,    _,    _,    true, _,    _     -> min x z
        |_,    _,    _,    _,    true, _     -> min y z
        |_,    _,    _,    _,    _,    true  -> min x y
                                               
        |_,    _,    _,    _,    _,    _     -> min x (min y z)

(* allonge la liste de Hamming de n lments *)
let rec allonge n h x y z =
  if n = 0 then h
  else begin
    let t = dmin x y z in
    h.tl <- t;
    allonge (n-1) t (if t =. x then double    x.tl.tl else x)
                    (if t =. y then triple    y.tl.tl else y)
                    (if t =. z then quintuple z.tl.tl else z)
  end

(* lance les calculs *)
let _ = 
  let rec h = {a=0; b=0; c=0; tl=h} in
  let n = int_of_string(Sys.argv.(1)) in
  let t = allonge n h (double h) (triple h) (quintuple h) in
  Printf.printf "2^%d . 3^%d . 5^%d" t.a t.b t.c;
  print_newline()
